<?php

namespace Smartcat\Includes\Services\Posts;

use Smartcat\Includes\Requests\PullPostsRequest;
use Smartcat\Includes\Services\Elementor\Document;
use Smartcat\Includes\Services\Interfaces\SmartcatPostsDatabaseInterface;
use Smartcat\Includes\Services\Interfaces\SmartcatPostTypeInterface;
use Smartcat\Includes\Services\Interfaces\SmartcatMetadataInterface;
use Smartcat\Includes\Services\Interfaces\SmartcatWpmlInterface;
use Smartcat\Includes\Services\Tools\SmartcatLogger;

class SmartcatPostsPull
{
    /**
     * @var SmartcatPostsDatabaseInterface
     */
    private $postsDatabaseService;

    /**
     * @var SmartcatWpmlInterface
     */
    private $smartcatWpmlService;

    /**
     * @var SmartcatMetadataInterface
     */
    private $metadataService;

    /** @var SmartcatPostTypeInterface */
    private $postTypesService;

    public function __construct(
        SmartcatPostsDatabaseInterface $postsDatabaseService,
        SmartcatWpmlInterface          $smartcatWpmlService,
        SmartcatMetadataInterface      $metadataService,
        SmartcatPostTypeInterface      $postTypeService
    )
    {
        $this->postsDatabaseService = $postsDatabaseService;
        $this->smartcatWpmlService = $smartcatWpmlService;
        $this->metadataService = $metadataService;
        $this->postTypesService = $postTypeService;
    }

    /**
     * @param PullPostsRequest $request
     * @return array
     * @throws \Exception
     */
    public function getPosts(PullPostsRequest $request): array
    {
        $this->smartcatWpmlService->switchLang($request->getLang());
        $args = $this->getArgs($request);
        $posts = $this->postsDatabaseService->getPosts($args);
        return $this->map($posts, $request->getMeta());
    }

    /**
     * @param PullPostsRequest $request
     * @return array
     */
    private function getArgs(PullPostsRequest $request): array
    {
        $args = [
            'post_type' => $this->postTypesService->getTranslatableTypes(),
            'numberposts' => $request->getLimit(),
            'offset' => $request->getOffset(),
            'suppress_filters' => false,
        ];

        if (!empty($request->getDate()) && empty($request->getIds())) {
            $args['date_query'] = [
                'column' => 'post_modified',
                'after' => $request->getDate(),
                'before' => current_time('Y-m-d H:i:s')
            ];
        }

        if (!empty($request->getIds())) {
            $args['numberposts'] = -1;
            $args['offset'] = 0;
            $args['include'] = implode(',', $request->getIds());
        }

        return $args;
    }

    /**
     * @throws \Exception
     */
    private function map(array $posts, array $metaKeys = []): array
    {
        return array_map(function ($post) use ($metaKeys) {

            $isElementorDocument = false;
            $elementorData = [];

            if (is_elementor_installed()) {
                try {
                    $elementorDocument = new Document($post->ID);
                    $isElementorDocument = $elementorDocument->isBuiltWithElementor();
                    $elementorData = $elementorDocument->getData();
                } catch (\Throwable $exception) {
                    $isElementorDocument = false;
                    smartcat_logger()->warn('Error Elementor post. Content will be exported of Gutenberg editor.', [
                        'postId' => $post->ID,
                        'error' => $exception->getMessage(),
                        'stackTrace' => $exception->getTraceAsString()
                    ]);
                }
            }

            return [
                'id' => $post->ID,
                'title' => $post->post_title,
                'postName' => $this->postsDatabaseService->normalizePostName($post->ID, $post->post_name),
                'slug' => urldecode($post->post_name),
                'content' => "",
                'acf' => [],
                'blocks' => $isElementorDocument
                    ? $elementorData
                    : SmartcatBlocks::parseContentBlocks($post->post_content),
                'type' => $post->post_type,
                'categories' => $this->postsDatabaseService->getCategoriesIds($post->ID),
                'metadata' => $this->metadataService->getFields($post->ID, $metaKeys)
            ];
        }, $posts);
    }
}