<?php

namespace Smartcat\Includes\Services\Posts;

use Smartcat\Includes\Requests\PushPostsRequest;
use Smartcat\Includes\Services\Elementor\Document;
use Smartcat\Includes\Services\Interfaces\SmartcatPostsDatabaseInterface;
use Smartcat\Includes\Services\Interfaces\SmartcatMetadataInterface;
use Smartcat\Includes\Services\Interfaces\SmartcatWpmlInterface;

class SmartcatPostsPush
{
    /**
     * @var SmartcatWpmlInterface
     */
    private $wpmlService;

    /**
     * @var SmartcatMetadataInterface
     */
    private $metadataService;

    /**
     * @var SmartcatPostsDatabaseInterface
     */
    private $postsDatabase;

    public function __construct(
        SmartcatWpmlInterface          $wpmlService,
        SmartcatMetadataInterface      $metadataService,
        SmartcatPostsDatabaseInterface $postsDatabase
    )
    {
        $this->wpmlService = $wpmlService;
        $this->metadataService = $metadataService;
        $this->postsDatabase = $postsDatabase;
    }

    public function push(PushPostsRequest $request)
    {
        foreach ($request->getPosts() as $post) {
            $targetPostId = $this->wpmlService->getTargetElementId(
                $post['originalId'],
                $post['targetLang'],
                "post_{$post['type']}"
            );

            if (!empty($targetPostId)) {
                $this->updatePost($targetPostId, $post);
            } else {
                $this->addPost($post, $request->getLang());
            }
        }
    }

    private function updatePost(int $postId, array $postData)
    {
        $originalPost = get_post($postData['originalId']);

        if (is_elementor_installed()) {
            $mayBeElementorOriginalPost = new Document($originalPost->ID);

            if ($mayBeElementorOriginalPost->isBuiltWithElementor()) {
                // updating target post data of original post
                (new Document($postId))->save(
                    $mayBeElementorOriginalPost->getElements(true)
                );

                $mayBeElementorTargetPost = new Document($postId);
                $mayBeElementorTargetPost->updateElementsData($postData['blocks']);
            } else {
                // FIXME: duplicated code
                $content = SmartcatBlocks::applyTranslatedBlocks(
                    $originalPost->post_content,
                    $postData['blocks']
                );

                $this->postsDatabase->updatePostContent($postId, $content);
            }
        } else {
            $content = SmartcatBlocks::applyTranslatedBlocks(
                $originalPost->post_content,
                $postData['blocks']
            );

            $this->postsDatabase->updatePostContent($postId, $content);
        }

        wp_update_post([
            'ID' => $postId,
            'post_title' => $postData['title'],
            'post_status' => $postData['postStatus'] ?? 'draft'
        ]);

        $this->metadataService->addMetaDataToPost($postId, $postData['metadata']);
    }

    private function addPost(array $postData, string $sourceLang)
    {
        $originalPost = get_post($postData['originalId']);

        $id = smartcat_wpml()->makeDuplicate($postData['originalId'], $postData['targetLang']);

        if (is_elementor_installed()) {
            $mayBeElementorPost = new Document($originalPost->ID);

            if ($mayBeElementorPost->isBuiltWithElementor()) {
                // updating target post data of original post
                (new Document($id))->save(
                    $mayBeElementorPost->getElements(true)
                );

                $targetPost = new Document($id);
                $targetPost->updateElementsData($postData['blocks']);
            } else {
                // FIXME: duplicated code
                $postData['content'] = SmartcatBlocks::applyTranslatedBlocks(
                    $originalPost->post_content,
                    $postData['blocks']
                );

                $this->postsDatabase->updatePostContent($id, $postData['content']);
            }
        } else {
            $postData['content'] = SmartcatBlocks::applyTranslatedBlocks(
                $originalPost->post_content,
                $postData['blocks']
            );

            $this->postsDatabase->updatePostContent($id, $postData['content']);
        }

        wp_update_post([
            'ID' => $id,
            'post_title' => $postData['title'],
            'post_status' => $postData['postStatus'] ?? 'draft'
        ]);

        $this->metadataService->addMetaDataToPost($id, $postData['metadata']);
    }

    private function handleCustomFields(int $postId, array $postData)
    {
        $this->metadataService->updateMetaDataByOriginalPost($postData['originalId'], $postId);
        $this->metadataService->addMetaDataToPost($postId, $postData['metadata']);
    }
}