<?php

namespace Smartcat\Includes\Services\TranslationRequest;

use Smartcat\Includes\Services\Smartcat\SmartcatClient;
use Smartcat\Includes\Services\Smartcat\SmartcatSmartcatHubClient;

class SmartcatDocumentsManager
{
    private $wpdb;
    private $postId;
    private $hubClient;
    private $smartcatClient;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->hubClient = new SmartcatSmartcatHubClient();
        $this->smartcatClient = new SmartcatClient();
    }

    public function getPostDocuments($postId = null)
    {
        return $this->wpdb->get_results(
            $this->wpdb->prepare(
                "SELECT * FROM {$this->getTableName()} WHERE post_id = %d", $postId
            )
        );
    }

    public function getTranslationRequestId($postId)
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT translation_request_id FROM {$this->getTableName()} WHERE post_id = %d LIMIT 1", [$postId]
            )
        )[0] ?? NULL;
    }

    public function create(array $attributes)
    {
        smartcat_logger()->info('Added new document to translation request', [
            'data' => $attributes
        ]);
        return $this->wpdb->insert($this->getTableName(), $attributes);
    }

    public function hasTranslationRequest($postId): bool
    {
        $count = $this->wpdb->get_var(
            $this->wpdb->prepare(
                "SELECT COUNT(*) FROM {$this->getTableName()} WHERE post_id = %d", $postId
            )
        );

        return $count > 0;
    }

    public function translationRequestExists($translationRequestId): bool
    {
        $count = $this->wpdb->get_var(
            $this->wpdb->prepare(
                "SELECT COUNT(*) FROM {$this->getTableName()} WHERE `translation_request_id` = %s", $translationRequestId
            )
        );

        return $count > 0;
    }

    public function updateDocumentsProgress($postId)
    {
        $documents = $this->getPostDocuments($postId);
        if (!empty($documents)) {
            $translationRequest = new SmartcatTranslationRequest($documents[0]->translation_request_id);
            $project = smartcat_hub_client()->getProject(
                $documents[0]->smartcat_project_id,
                $translationRequest->getSourceLanguageCode()
            );
            if (is_wp_error($project)) {
                return;
            }
            $projectDocuments = $project['documents'];
            foreach ($documents as $document) {
                $smartcatDocument = array_filter($projectDocuments, function ($d) use ($document) {
                    return $d['id'] === $document->smartcat_document_id;
                });
                if (empty($smartcatDocument)) {
                    smartcat_logger()->warn("Smartcat document '$document->smartcat_document_id' not found. Document will be deleted from translation request.", [
                        'smartcat_project_id' => $document->smartcat_project_id,
                        'smartcat_document_id' => $document->smartcat_document_id,
                        'translation_request_id' => $document->translation_request_id
                    ]);
                    $this->deleteDocument($document->smartcat_document_id);
                    continue;
                }
                $smartcatDocument = array_shift($smartcatDocument);

                $workflowStages = $smartcatDocument['workflowStages'];

                $maxProgress = count($workflowStages) * 100;
                $totalProgress = 0;

                foreach ($workflowStages as $workflowStage) {
                    $totalProgress += $workflowStage['progress'];
                }

                $progress = ($totalProgress / $maxProgress) * 100;

                $this->updateDocumentProgress($smartcatDocument['id'], $progress);
            }
        }
    }

    public function updateDocumentProgress($smartcatDocumentId, $progress)
    {
        $this->wpdb->update($this->getTableName(), [
            'translation_progress' => round($progress, 2)
        ], ['smartcat_document_id' => $smartcatDocumentId]);
    }

    public function getPostTranslatedLanguages($postId, $translationRequestId): array
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT lang FROM {$this->getTableName()} WHERE post_id = %d AND translation_request_id = %s", [$postId, $translationRequestId]
            )
        );
    }

    public function getPostProjectId($postId, $translationRequestId): string
    {
        $document = $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT smartcat_project_id FROM {$this->getTableName()} WHERE post_id = %d AND translation_request_id = %s LIMIT 1", [$postId, $translationRequestId]
            )
        );

        return array_shift($document);
    }

    public function setPostIdByDocumentId($documentId, $postId)
    {
        $this->wpdb->update($this->getTableName(), [
            'translated_post_id' => $postId
        ], ['smartcat_document_id' => $documentId]);
    }

    public function removeLanguage($postId, $language)
    {
        $this->wpdb->delete($this->getTableName(), [
            'post_id' => $postId,
            'lang' => $language
        ]);
    }

    public function getTranslationRequests(): array
    {
        $translationRequests = [];

        foreach ($this->getTranslationRequestIds() as $index => $translationRequestId) {
            $translationRequest = new SmartcatTranslationRequest($translationRequestId);

            $projectId = $translationRequest->getProjectId();

            if (empty($projectId)) {
                continue;
            }

            $smartcatProject = smartcat_hub_client()->getProject(
                $projectId,
                $translationRequest->getSourceLanguageCode()
            );

            foreach ($translationRequest->getPostIds() as $postId) {
                $postExists = get_post($postId);
                if (empty($postExists)) {
                    smartcat_dm()->removePost($postId);
                }
            }

            if (!is_wp_error($smartcatProject)) {
                foreach ($translationRequest->getPostIds() as $postId) {
                    $this->updateDocumentsProgress($postId);
                }
            }

            $translationRequests[] = [
                'index' => $index + 1,
                'id' => $translationRequestId,
                'name' => $translationRequest->getName(),
                'status' => !is_wp_error($smartcatProject) ? $translationRequest->getStatus() : [
                    'type' => 'error'
                ],
                'posts' => $translationRequest->getPosts(),
                'comment' => $translationRequest->getComment(),
                'project' => [
                    'isError' => is_wp_error($smartcatProject),
                    'id' => !is_wp_error($smartcatProject) ? $smartcatProject['id'] : NULL,
                    'name' => !is_wp_error($smartcatProject) ? $smartcatProject['name'] : NULL,
                    'deadline' => !is_wp_error($smartcatProject) ? $smartcatProject['deadline'] ?? NULL : NULL
                ],
                'languages' => [
                    'source' => $translationRequest->getSourceLanguage(),
                    'target' => $translationRequest->getTargetLanguages()
                ]
            ];
        }

        return [
            'items' => $translationRequests,
        ];
    }

    private function getTranslationRequestIds(): array
    {
        return $this->wpdb->get_col("SELECT distinct translation_request_id FROM {$this->getTableName()}");
    }

    public function deleteTranslationRequest($trId)
    {
        $this->wpdb->delete($this->getTableName(), [
            'translation_request_id' => $trId
        ]);
    }

    public function removePostFromTranslationRequest($postId, $translationRequestId)
    {
        $this->wpdb->delete($this->getTableName(), [
            'post_id' => $postId,
            'translation_request_id' => $translationRequestId
        ]);
    }

    public function removePost($postId)
    {
        $this->wpdb->delete($this->getTableName(), [
            'post_id' => $postId,
        ]);
    }

    public function deleteDocumentsByProjectId($projectId)
    {
        $this->wpdb->delete($this->getTableName(), [
            'smartcat_project_id' => $projectId,
        ]);
    }

    public function deleteDocument($smartcatDocumentId)
    {
        $this->wpdb->delete($this->getTableName(), [
            'smartcat_document_id' => $smartcatDocumentId,
        ]);
    }

    private function getTableName(): string
    {
        return $this->wpdb->prefix . SMARTCAT_DOCUMENTS_TABLE_NAME;
    }
}