<?php

namespace Smartcat\Includes\Services\TranslationRequest;

use Smartcat\Includes\Plugin\SmartcatPluginMultilingual as i18n;

class SmartcatTranslationRequest
{
    private $translationRequestId;
    private $wpdb;

    public function __construct($translationRequestId)
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->translationRequestId = $translationRequestId;
    }

    public function postHasTranslatedLang($postId, $lang): bool
    {
        $result = $this->wpdb->get_results(
            $this->wpdb->prepare(
                "SELECT * FROM {$this->getTableName()} WHERE translation_request_id = %s AND post_id = %d AND lang = %s",
                [$this->translationRequestId, $postId, $lang]
            )
        );

        return !empty($result);
    }

    public function getDocuments()
    {
        return $this->wpdb->get_results(
            $this->wpdb->prepare(
                "SELECT * FROM {$this->getTableName()} WHERE translation_request_id = %s", [$this->translationRequestId]
            )
        );
    }

    public function getPosts(): array
    {
        $postIds = $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT distinct post_id FROM {$this->getTableName()} WHERE translation_request_id = %s", [$this->translationRequestId]
            )
        );
        $posts = [];
        foreach ($postIds as $postId) {
            $post = get_post($postId);
            $posts[$post->ID] = $post->post_title;
        }
        return $posts;
    }

    public function getPostIds(): array
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT distinct post_id FROM {$this->getTableName()} WHERE translation_request_id = %s", [$this->translationRequestId]
            )
        );
    }

    public function getName(): string
    {
        $createdAt = $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT created_at FROM {$this->getTableName()} WHERE translation_request_id = %s LIMIT 1", [$this->translationRequestId]
            )
        )[0];
        return i18n::_e('Translation request from ') . (new \DateTime($createdAt))->format('Y/m/d');
    }

    public function getStatus(): array
    {
        $documents = $this->getDocuments();
        $status = i18n::_e('Sent');

        $totalProgress = count($documents) * 100;
        $currentProgress = 0;

        foreach ($documents as $document) {
            $currentProgress += $document->translation_progress;
        }

        if ($currentProgress > 0 && $currentProgress < $totalProgress) {
            $status = i18n::_e('In progress');
        } else if ($currentProgress >= $totalProgress) {
            $status = i18n::_e('Completed');
        }

        return [
            'type' => $status,
            'progress' => $status === i18n::_e('In progress') ? round(($currentProgress / $totalProgress) * 100) : NULL
        ];
    }

    public function updateDocumentsProgress()
    {

    }

    public function getProjectId()
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT smartcat_project_id FROM {$this->getTableName()} WHERE translation_request_id = %s LIMIT 1", [$this->translationRequestId]
            )
        )[0];
    }

    public function getComment()
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT `comment` FROM {$this->getTableName()} WHERE translation_request_id = %s AND `comment` IS NOT NULL LIMIT 1", [$this->translationRequestId]
            )
        )[0] ?? NULL;
    }

    public function getCreatedAt(): string
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT created_at FROM {$this->getTableName()} WHERE translation_request_id = %s LIMIT 1", [$this->translationRequestId]
            )
        )[0];
    }

    public function getId(): string
    {
        return $this->translationRequestId;
    }

    public function getTargetLanguages(): array
    {
        $languages = [];
        $wpmlLanguages = smartcat_wpml()->getActiveLanguages();
        foreach ($this->getDocuments() as $document) {
            foreach ($wpmlLanguages as $wpmlLanguage) {
                if ($wpmlLanguage['code'] === $document->lang) {
                    $languages[$document->lang] = $wpmlLanguage['translated_name'];
                    break;
                }
            }
        }
        return $languages;
    }

    public function getSourceLanguage(): string
    {
        return smartcat_wpml()->getPostLanguageName($this->getDocuments()[0]->post_id);
    }

    public function getSourceLanguageCode(): string
    {
        return smartcat_wpml()->getPostLanguageCode($this->getDocuments()[0]->post_id);
    }

    private function getTableName(): string
    {
        return $this->wpdb->prefix . SMARTCAT_DOCUMENTS_TABLE_NAME;
    }
}
