<?php

namespace Smartcat\Includes\Services\Metabox;

use Smartcat\Includes\Services\Elementor\Document;
use Smartcat\Includes\Services\Interfaces\SmartcatMetadataInterface;
use Smartcat\Includes\Services\Posts\SmartcatBlocks;
use Smartcat\Includes\Services\Posts\SmartcatImportTranslations;
use Smartcat\Includes\Services\Smartcat\SmartcatHubClientInterface;
use Smartcat\Includes\Services\Smartcat\SmartcatClientInterface;
use Smartcat\Includes\Services\TranslationRequest\SmartcatDocumentsManager;
use Smartcat\Includes\Services\TranslationRequest\SmartcatTranslationRequest;

class SmartcatMetabox
{
    /**
     * @var SmartcatClientInterface
     */
    private $smartcatClient;

    /**
     * @var SmartcatHubClientInterface
     */
    private $hubClient;

    /** @var SmartcatDocumentsManager */
    private $documentsManager;

    private $importTranslationsService;

    /** @var SmartcatMetadataInterface $metadataService */
    private $metadataService;

    public function __construct(
        SmartcatClientInterface    $smartcatClient,
        SmartcatHubClientInterface $hubClient,
        SmartcatDocumentsManager   $documentsManager,
        SmartcatImportTranslations $importTranslationsService,
        SmartcatMetadataInterface  $metadataService
    )
    {
        $this->smartcatClient = $smartcatClient;
        $this->hubClient = $hubClient;
        $this->documentsManager = $documentsManager;
        $this->importTranslationsService = $importTranslationsService;
        $this->metadataService = $metadataService;
    }

    public function displayForm($post)
    {
        (new \Smartcat\Includes\Views\SmartcatMetabox($post->ID))->display();
    }

    public function logInToSmartcat()
    {
        $this->smartcatClient->logInRedirect();
    }

    public function createTranslationRequest($postId, $sourceLanguage, $targetLanguages, $projectId, $deadline, $comment): array
    {
        smartcat_logger()->info('Started the process of creating a translation request', [
            'post_id' => $postId,
            'source_language' => $sourceLanguage,
            'traget_languages' => $targetLanguages,
            'smartcat_project_id' => $projectId,
            'deadline' => $deadline
        ]);

        $translationRequestCreatedAt = date('Y-m-d H:i:s');
        $translationRequestUuid = $this->generateTranslationRequestUuid();
        $project = NULL;

        $postIds = $this->mapPostIds($postId);

        if (empty($projectId)) {
            $post = get_post($postIds[0]);
            $andMore = count($postIds) > 1 ? ' and more' : '';

            $project = $this->createSmartcatProject(
                "Translation request ({$post->post_title}$andMore)",
                $sourceLanguage,
                $targetLanguages,
                $deadline
            );

            if (is_wp_error($project)) {
                return $this->responseError($project);
            }

            $projectId = $project['id'];

            smartcat_logger()->info("Created new Smartcat project: $projectId");
        }

        if (!empty($deadline)) {
            if (empty($project)) {
                $project = $this->hubClient->getProject($projectId, $sourceLanguage);
            }
            $this->smartcatClient->updateProjectDeadline(
                $projectId,
                $project['name'],
                $deadline
            );
        }

        $smartcatDocumentIds = [];

        foreach ($postIds as $id) {
            $post = get_post($id);

            foreach ($targetLanguages as $language) {
                smartcat_logger()->info('Started import post to Smartcat', [
                    'post_id' => $postId,
                    'source_language' => $sourceLanguage,
                    'target_language' => $language,
                    'smartcat_project_id' => $projectId,
                ]);
                $data = $this->importTranslationsToSmartcat($post, $projectId, $sourceLanguage, $language);

                if (is_wp_error($data)) {
                    // TODO: выдать больше информации клиенту
                    return $this->responseError($data);
                }

                // TODO: сделать проверку на существующий документ
                $this->documentsManager->create([
                    'translated_post_id' => smartcat_wpml()->getTargetElementId($id, $language, 'post_' . get_post_type($id)),
                    'post_id' => $id,
                    'smartcat_document_id' => $data['documentId'],
                    'smartcat_project_id' => $projectId,
                    'lang' => $language,
                    'translation_progress' => 0,
                    'created_at' => $translationRequestCreatedAt,
                    'translation_request_id' => $translationRequestUuid,
                    'comment' => $comment
                ]);

                $smartcatDocumentIds[] = $data['documentId'];
            }
        }

        if (!empty($comment)) {
            $smartcatDocumentIds = array_map(function ($d) {
                return explode('_', $d)[0];
            }, $smartcatDocumentIds);
            $smartcatDocumentIds = array_unique($smartcatDocumentIds, SORT_STRING);
            foreach ($smartcatDocumentIds as $smartcatDocumentId) {
                $this->hubClient->addCommentToDocument($smartcatDocumentId, $comment);
            }
        }

        return $this->responseOk();
    }

    public function getTranslations($postIds, $translationRequestId = NULL): array
    {
        smartcat_logger()->info('Start getting translations for a translation request', [
            'post_ids' => $postIds,
            'translation_request_id' => $translationRequestId
        ]);

        if (empty($postIds)) {
            smartcat_logger()->info('Getting translations will be for the entire translation request', [
                'translation_request_id' => $translationRequestId
            ]);
            $postIds = (new SmartcatTranslationRequest($translationRequestId))->getPostIds();
        } else {
            $postIds = $this->mapPostIds($postIds);
        }

        foreach ($postIds as $postId) {
            $languages = $this->documentsManager->getPostTranslatedLanguages($postId, $translationRequestId);
            $projectId = $this->documentsManager->getPostProjectId($postId, $translationRequestId);

            smartcat_logger()->info('Getting translations for post', [
                'post_id' => $postId,
                'languages' => $languages,
                'smartcat_project_id' => $projectId
            ]);

            foreach ($languages as $lang) {
                $lang = sc_locale()->map($lang);

                $exportBegin = $this->hubClient->exportBegin($postId, $projectId, $lang);

                if (is_wp_error($exportBegin)) {
                    return $this->responseError($exportBegin);
                }

                if ($exportBegin['exportInfo']['exportStatus'] === 4) {
                    smartcat_logger()->warn("An error occurred during export begin. Language '$lang' will be skipped.", [
                        'translation_request_id' => $translationRequestId,
                        'post_id' => $postId,
                        'language' => $lang,
                        'smartcat_project_id' => $projectId
                    ]);
                    continue;
                }

                while (true) {
                    $exportResult = $this->hubClient->exportResult($exportBegin['exportInfo']);

                    if (is_wp_error($exportResult)) {
                        return $this->responseError($exportResult);
                    }

                    if (!empty($exportResult['items'])) {
                        smartcat_logger()->info('Start importing received translations from Smartcat to WordPress', [
                            'post_id' => $postId,
                            'language' => $lang,
                            'exported_items' => $exportResult['items']
                        ]);
                        $translatedPostId = $this->importTranslationsService->import($postId, $lang, $exportResult['items']);
                        $smartcatDocumentId = $exportResult['exportInfo']['smartcatDocumentId'];
                        $this->documentsManager->setPostIdByDocumentId($smartcatDocumentId, $translatedPostId);
                        smartcat_logger()->info("Document $smartcatDocumentId is assigned post $translatedPostId");
                        break;
                    }
                }
            }
        }

        return $this->responseOk();
    }

    public function addLanguageToTranslationRequest($postId, $translationRequestId, $language): array
    {
        smartcat_logger()->info('Adding new language to translation request', [
            'post_id' => $postId,
            'translation_request_id' => $translationRequestId,
            'target_language' => $language
        ]);

        $tr = new SmartcatTranslationRequest($translationRequestId);
        $data = $this->importTranslationsToSmartcat(get_post($postId), $tr->getProjectId(), $tr->getSourceLanguageCode(), $language);

        if (is_wp_error($data)) {
            return $this->responseError($data);
        }

        $this->documentsManager->create([
            'translated_post_id' => smartcat_wpml()->getTargetElementId($postId, $language, 'post_' . get_post_type($postId)),
            'post_id' => $postId,
            'smartcat_document_id' => $data['documentId'],
            'smartcat_project_id' => $tr->getProjectId(),
            'lang' => $language,
            'translation_progress' => 0,
            'created_at' => $tr->getCreatedAt(),
            'translation_request_id' => $tr->getId()
        ]);

        return $this->responseOk();
    }

    public function removePostFromTranslationRequest($postId, $translationRequestId): array
    {
        smartcat_logger()->info('Removing post from translation request', [
            'post_id' => $postId,
            'translation_request_id' => $translationRequestId,
        ]);
        $this->documentsManager->removePostFromTranslationRequest($postId, $translationRequestId);
        return $this->responseOk();
    }

    public function removeLanguageFromTranslationRequest($postId, $language): array
    {
        smartcat_logger()->info('Removing language from translation request', [
            'post_id' => $postId,
            'target_language' => $language,
        ]);
        $this->documentsManager->removeLanguage($postId, $language);
        return $this->responseOk();
    }

    public function removeTranslationRequest($trId): array
    {
        smartcat_logger()->info('Removing translation request', [
            'translation_request_id' => $trId,
        ]);
        $this->documentsManager->deleteTranslationRequest($trId);
        return $this->responseOk();
    }

    public function updateSourceContent($postIds, $translationRequestId): array
    {
        smartcat_logger()->info('Update source document content', [
            'post_ids' => $postIds,
            'translation_request_id' => $translationRequestId
        ]);
        $postIds = !empty($postIds) ? $this->mapPostIds($postIds) : NULL;

        $translationRequest = new SmartcatTranslationRequest($translationRequestId);
        $postIds = empty($postIds) ? $translationRequest->getPostIds() : $postIds;
        $projectId = $translationRequest->getProjectId();
        $sourceLanguage = $translationRequest->getSourceLanguageCode();

        foreach ($postIds as $postId) {
            $post = get_post($postId);
            $documents = smartcat_dm()->getPostDocuments($postId);
            foreach ($documents as $document) {
                $this->importTranslationsToSmartcat($post, $projectId, $sourceLanguage, $document->lang);
            }
        }

        return $this->responseOk();
    }

    private function createSmartcatProject($name, $sourceLanguage, $targetLanguages, $deadline)
    {
        return $this->hubClient->createProject(
            $name,
            $sourceLanguage,
            $targetLanguages
        );
    }

    private function mapPostIds($postIds)
    {
        return is_numeric($postIds) ? [$postIds] : json_decode($postIds, true);
    }

    private function importTranslationsToSmartcat($post, $projectId, $sourceLanguage, $targetLanguage)
    {
        return $this->hubClient->importTranslations(
            $post->post_title,
            "$post->ID",
            $projectId,
            $sourceLanguage,
            $targetLanguage,
            $this->getPostData($post)
        );
    }

    private function getPostData($post): array
    {
        $data = [
            [
                'id' => 'post_title',
                'sourceText' => $post->post_title,
                'format' => 'plain',
                'properties' => [
                    'context' => 'title'
                ],
                'existingTranslation' => ''
            ]
        ];

        $isElementorDocument = false;
        $elementorData = [];

        if (is_elementor_installed()) {
            try {
                $elementorDocument = new Document($post->ID);
                $isElementorDocument = $elementorDocument->isBuiltWithElementor();
                $elementorData = $elementorDocument->getData();
            } catch (\Throwable $exception) {
                $isElementorDocument = false;
                smartcat_logger()->warn('Error Elementor post. Content will be exported of Gutenberg editor.', [
                    'postId' => $post->ID,
                    'error' => $exception->getMessage(),
                    'stackTrace' => $exception->getTraceAsString()
                ]);
            }
        }

        $metadata = $this->metadataService->getFields($post->ID);

        if ($isElementorDocument) {
            $data = array_merge(
                $data,
                $this->mapElementorData($elementorData),
                $this->mapMataData($metadata)
            );
        } else {
            $blocks = SmartcatBlocks::parseContentBlocks($post->post_content);

            if (sc_built_with_bakery($post->ID)) {
                $blocks = array_map(function ($block) {
                    $block['content'] = sc_bakery_builder()->decode($block['content']);
                    return $block;
                }, $blocks);
            }

            $data = array_merge(
                $data,
                $this->mapMataData($metadata),
                $this->mapBlocks($blocks)
            );
        }

        return array_values($this->filterData($data));
    }

    private function filterData($data): array
    {
        return array_filter($data, function ($item) {
            return !empty($item['sourceText']);
        });
    }

    private function mapBlocks($data): array
    {
        return array_map(function ($item) {
            return [
                'id' => $item['key'],
                'sourceText' => $item['content'],
                'format' => 'auto',
                'comments' => [],
                'properties' => [
                    'context' => 'block'
                ],
                'existingTranslation' => ''
            ];
        }, $data);
    }

    private function mapMataData($data): array
    {
        $mappedData = [];
        foreach ($data as $key => $value) {
            $mappedData[] = [
                'id' => $key,
                'sourceText' => $value['content'],
                'format' => 'auto',
                'properties' => [
                    'context' => 'metadata',
                ],
                'existingTranslation' => ''
            ];
        }
        return $mappedData;
    }

    private function mapElementorData($data): array
    {
        $mappedData = [];
        foreach ($data as $element) {
            $mappedData[] = [
                'id' => $element['key'],
                'sourceText' => $element['content'],
                'format' => 'auto',
                'properties' => [
                    'context' => 'elementor',
                ],
                'existingTranslation' => ''
            ];
        }
        return $mappedData;
    }

    private function generateTranslationRequestUuid(): string
    {
        return uniqid('tr_');
    }

    private function responseError($e): array
    {
        return [
            'status' => 'failed',
            'message' => $e->get_error_message()
        ];
    }

    private function responseOk(): array
    {
        return [
            'status' => 'success'
        ];
    }
}