<?php

namespace Smartcat\Includes\Services\Smartcat;

use Smartcat\Includes\Services\Errors\SmartcatWpError;
use WP_Http;

class SmartcatClient implements SmartcatClientInterface
{
    public function logInRedirect()
    {
        $url = self::getAuthHost() . "/sign-in?backUrl=/integrations/wpml-sign-in?wordpressUrl=" . get_site_url();
        header("Location: $url");
    }

    public static function isAuthorized(): bool
    {
        return !empty(get_option('smartcat_account_id'))
            && !empty(get_option('smartcat_api_key'))
            && !empty(get_option('smartcat_hub_key'))
            && !empty(get_option('smartcat_api_host'))
            && !empty(get_option('smartcat_hub_host'));
    }

    public function createProject($postId, $sourceLanguage, $targetLanguages)
    {
        $data = [
            'name' => "WPML Integration: post - $postId",
            'sourceLanguage' => $sourceLanguage,
            'targetLanguages' => $targetLanguages,
            'enableProjectTasks' => true,
            'externalTag' => 'ihub-wpml'
        ];

        $response = $this->sendRequest('api/integration/v1/project/create', ['value' => json_encode($data)]);

        $responseData = json_decode($response['body'], true);

        return $responseData['id'] ?? NULL;
    }

    public function updateProjectDeadline($projectId, $projectName, $deadline)
    {
        $data = [
            'name' => $projectName,
            'deadline' => $deadline
        ];

        $response = $this->sendRequest("api/integration/v1/project/$projectId", json_encode($data), 'PUT', [
            'Content-type' => 'application/json'
        ]);

        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error update project deadline', [
                'smartcat_project_id' => $projectId,
                'api_host' => $this->getHost(),
                'api_endpoint' => "api/integration/v1/project/$projectId",
                'request_data' => $data,
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error update project deadline');
        }
    }

    public function getProject($projectId)
    {
        $response = $this->sendRequest("api/integration/v1/project/$projectId", [], 'GET');
        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error loading project from Smartcat account', [
                'smartcat_project_id' => $projectId,
                'api_host' => $this->getHost(),
                'response' => $response['response'],
                'api_endpoint' => "api/integration/v1/project/$projectId",
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);

            if ($response['response']['code'] === 404 || $response['response']['code'] === 403) {
                smartcat_logger()->warn("Project '$projectId' not found or account have not access. Translation requests with this project will be deleted.");
                smartcat_dm()->deleteDocumentsByProjectId($projectId);
            }

            return new SmartcatWpError(500, 'Error loading project from Smartcat account');
        }

        return $this->getJsonFromBody($response);
    }

    /**
     * @throws \Exception
     */
    public function getAccountProjects()
    {
        $response = $this->sendRequest('api/integration/v1/project/list', [], 'GET');
        if (!$this->checkResponse($response)) {
            return new SmartcatWpError(500, 'Error loading project list from Smartcat account');
        }
        return $this->getJsonFromBody($response);
    }

    public function redirectToProject($projectId)
    {
        header('Location: ' . "{$this->getHost()}projects/$projectId/integrations");
    }

    public static function getAuthHost()
    {
        if (empty(get_option('smartcat_auth_host'))) {
            return self::debugMode() ? SMARTCAT_HOST_DEV : SMARTCAT_HOST;
        }
        return get_option('smartcat_auth_host');
    }

    public static function debugMode()
    {
        if (empty(get_option('smartcat_debug_mode'))) {
            return SMARTCAT_DEV_MODE;
        }
        return (bool)get_option('smartcat_debug_mode');
    }

    private function sendRequest($uri, $body = [], $method = 'POST', $headers = [])
    {
        $http = new WP_Http();
        return $http->request($this->getHost() . $uri, [
            'method' => $method,
            'timeout' => 30,
            'body' => $body,
            'headers' => array_merge([
                'Authorization' => $this->getAuthorizationToken(),
            ], $headers)
        ]);
    }

    private function getAuthorizationToken(): string
    {
        return 'Basic ' . base64_encode(get_option('smartcat_account_id') . ':' . get_option('smartcat_api_key'));
    }

    private function getHost(): string
    {
        return get_option('smartcat_api_host');
    }

    private function checkResponse($response): bool
    {
        if ($response instanceof \WP_Error) {
            return false;
        }

        if (isset($response['response'])) {
            preg_match('/20[01]/', $response['response']['code'], $output);
            return !empty($output);
        }

        return false;
    }

    private function getJsonFromBody($response)
    {
        return json_decode($response['body'], true);
    }
}