<?php

namespace Smartcat\Includes\Services\Smartcat;

use Smartcat\Includes\Services\Errors\SmartcatWpError;
use WP_Http;

class SmartcatSmartcatHubClient implements SmartcatHubClientInterface
{
    public function createIntegration($scAccountId, $scAccountKeyId, $projectId, $sourceLanguage, $targetLanguages, $url, $secret, $postIds = [], $wordPressMetadataKeys = [], $isPublicPosts = 'publish')
    {
        $data = [
            'name' => 'WPML Integration',
            'isUserConfirmedLanguages' => false,
            'scAccountId' => $scAccountId,
            'scAccountKeyId' => $scAccountKeyId,
            'scProjectId' => $projectId,
            'sourceLanguage' => $sourceLanguage,
            'targetLanguages' => $targetLanguages,
            'integrationType' => 1,
            'adapterParams' => [
                'wpmlParams' => [
                    'wordPressUrl' => $url,
                    'wordPressPluginKey' => $secret,
                    'wordPressPublicPosts' => $isPublicPosts,
                    'wordPressPostIds' => $postIds,
                    'wordPressMetadataKeys' => $wordPressMetadataKeys
                ]
            ],
            'flowCompleted' => true,
            'externalTag' => 'ihub-wpml'
        ];
        $response = $this->sendRequest('api/integrations', json_encode($data));
        $responseData = json_decode($response['body'], true);

        return $responseData['id'] ?? NULL;
    }

    public function startIntegration($integrationId, $mode = 3)
    {
        $data = [
            'mode' => $mode,
            'integrationId' => $integrationId
        ];
        $this->sendRequest('api/integrations/start', json_encode($data));
    }

    public function createProject(string $name, string $sourceLanguage, array $sourceLanguages)
    {
        $data = [
            'scProjectName' => $name,
            'scAccountId' => get_option('smartcat_account_id'),
            'sourceLanguage' => $sourceLanguage,
            'targetLanguages' => $sourceLanguages,
        ];

        $response = $this->sendRequest('api/wpml-app/project', json_encode($data));

        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error when creating a project in Smartcat', [
                'request_data' => $data,
                'api_host' => $this->getHost(),
                'api_endpoint' => 'api/wpml-app/project',
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error when creating a project in Smartcat');
        }

        return $this->getJsonFromBody($response);
    }

    public function importTranslations($postName, $postId, $projectId, $sourceLang, $targetLang, $items)
    {
        $data = [
            'wpmlPostName' => $postName,
            'wpmlPostId' => $postId,
            'scAccountId' => get_option('smartcat_account_id'),
            'scProjectId' => $projectId,
            'sourceLanguage' => $sourceLang,
            'targetLanguage' => $targetLang,
            'items' => $items
        ];

        $response = $this->sendRequest('api/wpml-app/import', json_encode($data));

        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error when importing translations', [
                'request_data' => $data,
                'api_host' => $this->getHost(),
                'api_endpoint' => 'api/wpml-app/import',
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error when importing translations');
        }

        return $this->getJsonFromBody($response);
    }

    public function getProject($projectId, $sourceLanguage)
    {
        $data = [
            'scProjectId' => $projectId,
            'scAccountId' => get_option('smartcat_account_id'),
            'sourceLanguage' => $sourceLanguage
        ];

        $response = $this->sendRequest('api/wpml-app/project', json_encode($data));

        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error getting smartcat project', [
                'smartcat_project_id' => $projectId,
                'source_language' => $sourceLanguage,
                'request_data' => $data,
                'api_host' => $this->getHost(),
                'api_endpoint' => 'api/wpml-app/project',
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error when creating a project in Smartcat');
        }

        return $this->getJsonFromBody($response);
    }

    public function exportBegin($postId, $projectId, $targetLanguage)
    {
        $data = [
            'wpmlPostId' => $postId,
            'targetLanguage' => $targetLanguage,
            'scProjectId' => $projectId,
            'scAccountId' => get_option('smartcat_account_id')
        ];

        $response = $this->sendRequest('api/wpml-app/export-begin', json_encode($data));

        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error getting export begin data', [
                'post_id' => $postId,
                'smartcat_project_id' => $projectId,
                'target_language' => $targetLanguage,
                'request_data' => $data,
                'api_host' => $this->getHost(),
                'api_endpoint' => 'api/wpml-app/export-begin',
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error export documents from Smartcat');
        }

        return $this->getJsonFromBody($response);
    }

    public function exportResult(array $exportInfo)
    {
        $response = $this->sendRequest('api/wpml-app/export-result', json_encode($exportInfo));

        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error export result', [
                'request_data' => $exportInfo,
                'api_host' => $this->getHost(),
                'api_endpoint' => 'api/wpml-app/export-result',
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error export documents from Smartcat');
        }

        return $this->getJsonFromBody($response);
    }

    public function addCommentToDocument($documentId, $comment)
    {
        $data = ['commentText' => $comment];
        $response = $this->sendRequest("api/wpml-app/add-document-comment/$documentId", json_encode($data));
        if (!$this->checkResponse($response)) {
            smartcat_logger()->error('Error add comment to document', [
                'request_data' => $data,
                'api_host' => $this->getHost(),
                'api_endpoint' => "api/wpml-app/add-document-comment/$documentId",
                'response_body' => is_array($response) ? $response['body'] : $response,
                'response_errors' => $response instanceof \WP_Error ? $response->get_error_messages() : NULL
            ]);
            return new SmartcatWpError(500, 'Error add comment to document');
        }
        return $this->getJsonFromBody($response);
    }

    private function sendRequest($uri, $body = [], $method = 'POST')
    {
        $http = new WP_Http();
        return $http->request($this->getHost() . $uri, [
            'method' => $method,
            'timeout' => 30,
            'body' => $body,
            'headers' => [
                'Authorization' => $this->getAuthorizationToken(),
                'Content-Type' => 'application/json'
            ]
        ]);
    }

    private function getAuthorizationToken(): string
    {
        return 'SmartcatApi ' . base64_encode(get_option('smartcat_account_id') . ':' . get_option('smartcat_api_key'));
    }

    private function getHost(): string
    {
        $length = strlen('/');

        $host = get_option('smartcat_hub_host');

        if (empty($host)) {
            $host = SMARTCAT_IHUB_HOST;
        }

        if (substr($host, -$length) === '/') {
            return $host;
        }

        return $host . '/';
    }

    private function checkResponse($response): bool
    {
        if ($response instanceof \WP_Error) {
            return false;
        }

        if (isset($response['response'])) {
            preg_match('/20[01]/', $response['response']['code'], $output);
            return !empty($output);
        }

        return false;
    }

    private function getJsonFromBody($response)
    {
        return json_decode($response['body'], true);
    }
}